'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  var keysA = Object.keys(objA);

  if (keysA.length !== Object.keys(objB).length) {
    return false;
  }

  for (var i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

var reducer = function reducer(state, newState) {
  return newState;
};

var useIsoLayoutEffect = typeof window !== 'undefined' ? react.useLayoutEffect : react.useEffect;
function create(createState) {
  var listeners = new Set();

  var setState = function setState(partial) {
    var partialState = typeof partial === 'function' ? partial(state) : partial;

    if (partialState !== state) {
      state = Object.assign({}, state, partialState);
      listeners.forEach(function (listener) {
        return listener(state);
      });
    }
  };

  var getState = function getState() {
    return state;
  }; // Optional selector param goes first so we can infer its return type and use
  // it for listener


  var subscribe = function subscribe(selectorOrListener, listenerOrUndef, equalityFn) {
    var listener = selectorOrListener; // Existance of second param means a selector was passed in

    if (listenerOrUndef) {
      // We know selector is not type StateListener so it must be StateSelector
      var selector = selectorOrListener;
      var stateSlice = selector(state);

      listener = function listener() {
        try {
          var sel = selector(state);
          var old = stateSlice; // Update local state slice

          stateSlice = sel; // Test for changes

          var equal = equalityFn ? equalityFn(old, sel) : old === sel; // Call listeners if state has changed

          if (!equal) listenerOrUndef(stateSlice);
        } catch (_unused) {}
      };
    }

    listeners.add(listener);
    return function () {
      return void listeners["delete"](listener);
    };
  };

  var destroy = function destroy() {
    listeners.clear();
  };

  var useStore = function useStore(selector, equalityFn) {
    var selRef = react.useRef(selector);

    var _useReducer = react.useReducer(reducer, state, // Optional third argument but required to not be 'undefined'
    selector),
        stateSlice = _useReducer[0],
        dispatch = _useReducer[1]; // Need to manually get state slice if selector has changed with no deps or
    // deps exist and have changed


    if (selector && selector !== selRef.current) stateSlice = selector(state); // Update refs synchronously after view has been updated

    useIsoLayoutEffect(function () {
      return void (selRef.current = selector);
    }, [selector]); // Subscribe to the store

    useIsoLayoutEffect(function () {
      return selector ? subscribe( // Truthy check because it might be possible to set selRef to
      // undefined and call this subscriber before it resubscribes
      function () {
        return selRef.current ? selRef.current(state) : state;
      }, dispatch, equalityFn) : subscribe(dispatch); // Only resubscribe to the store when changing selector from function to
      // undefined or undefined to function
    }, [!selector]);
    return stateSlice;
  };

  var api = {
    destroy: destroy,
    getState: getState,
    setState: setState,
    subscribe: subscribe
  };
  var state = createState(setState, getState, api);
  return [useStore, api];
}

exports.default = create;
exports.shallowEqual = shallowEqual;
