import { useRef, useReducer, useLayoutEffect, useEffect } from 'react';

function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  const keysA = Object.keys(objA);

  if (keysA.length !== Object.keys(objB).length) {
    return false;
  }

  for (let i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

const reducer = (state, newState) => newState;

const useIsoLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect : useEffect;
function create(createState) {
  const listeners = new Set();

  const setState = partial => {
    const partialState = typeof partial === 'function' ? partial(state) : partial;

    if (partialState !== state) {
      state = Object.assign({}, state, partialState);
      listeners.forEach(listener => listener(state));
    }
  };

  const getState = () => state; // Optional selector param goes first so we can infer its return type and use
  // it for listener


  const subscribe = (selectorOrListener, listenerOrUndef, equalityFn) => {
    let listener = selectorOrListener; // Existance of second param means a selector was passed in

    if (listenerOrUndef) {
      // We know selector is not type StateListener so it must be StateSelector
      const selector = selectorOrListener;
      let stateSlice = selector(state);

      listener = () => {
        try {
          const sel = selector(state);
          const old = stateSlice; // Update local state slice

          stateSlice = sel; // Test for changes

          const equal = equalityFn ? equalityFn(old, sel) : old === sel; // Call listeners if state has changed

          if (!equal) listenerOrUndef(stateSlice);
        } catch {}
      };
    }

    listeners.add(listener);
    return () => void listeners.delete(listener);
  };

  const destroy = () => {
    listeners.clear();
  };

  const useStore = (selector, equalityFn) => {
    const selRef = useRef(selector);
    let [stateSlice, dispatch] = useReducer(reducer, state, // Optional third argument but required to not be 'undefined'
    selector); // Need to manually get state slice if selector has changed with no deps or
    // deps exist and have changed

    if (selector && selector !== selRef.current) stateSlice = selector(state); // Update refs synchronously after view has been updated

    useIsoLayoutEffect(() => void (selRef.current = selector), [selector]); // Subscribe to the store

    useIsoLayoutEffect(() => {
      return selector ? subscribe( // Truthy check because it might be possible to set selRef to
      // undefined and call this subscriber before it resubscribes
      () => selRef.current ? selRef.current(state) : state, dispatch, equalityFn) : subscribe(dispatch); // Only resubscribe to the store when changing selector from function to
      // undefined or undefined to function
    }, [!selector]);
    return stateSlice;
  };

  let api = {
    destroy,
    getState,
    setState,
    subscribe
  };
  let state = createState(setState, getState, api);
  return [useStore, api];
}

export default create;
export { shallowEqual };
